/* 
   SQLServerContext.m

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author: Ovidiu Predescu <ovidiu@bx.logicnet.ro>
   Author: Scott Christley <scottc@net-community.com>
   Date: October 1996

   This file is part of the GNUstep Database Library.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <Foundation/NSString.h>
#include <Foundation/NSArray.h>
#include <Foundation/NSValue.h>
#include <extensions/NSException.h>
#include <eoaccess/exceptions/EOFExceptions.h>
#include "SQLServerContext.h"
#include "SQLServerChannel.h"

@implementation SQLServerContext

- (void)channelDidInit:aChannel
{
    if ([channels count])
	THROW([TooManyOpenedChannelsException new]);
    [super channelDidInit:aChannel];
}

- (BOOL)primaryBeginTransaction
{
    return [[[channels objectAtIndex:0] nonretainedObjectValue]
		evaluateExpression:@"BEGIN TRANSACTION"];
}

- (BOOL)primaryCommitTransaction
{
    return [[[channels objectAtIndex:0] nonretainedObjectValue]
		evaluateExpression:@"COMMIT TRANSACTION"];
}

- (BOOL)primaryRollbackTransaction
{
    return [[[channels objectAtIndex:0] nonretainedObjectValue]
		evaluateExpression:@"ROLLBACK TRANSACTION"];
}

- (BOOL)canNestTransactions		{ return YES; }

@end /* SQLServerContext */
