/* 
   PXKApplication.m

   NSApplication for GNUstep GUI X/DPS Backend

   The NSApplication class manages the main event loop of
   the program, keeps track of the application's windows
   and which one is the key window.

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author:  Pascal Forget <pascal@wsc.com>
   Date: January 1996
   
   This file is part of the GNUstep GUI X/DPS Backend.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <gnustep/xdps/PXKApplication.h>
#include <gnustep/xdps/PXKEventFactory.h>
#include <gnustep/xdps/PXKDPSContext.h>
#include <gnustep/xdps/PXKScreen.h>

extern NSEvent *NullEvent;

//
// Backend structure for PXKApplication
//
typedef struct _PXKApp_struct
{
  Window xWindow;
  GC context;
  Window grab_window;
} PXKApp_struct;

#define XWINDOW (((PXKApp_struct *)be_app_reserved)->xWindow)
#define XGC (((PXKApp_struct *)be_app_reserved)->context)
#define XGRAB (((PXKApp_struct *)be_app_reserved)->grab_window)

@implementation PXKApplication

- init
{
  // Allocate backend structure
  be_app_reserved = malloc(sizeof(PXKApp_struct));
  XWINDOW = 0;
  XGRAB = 0;

  return [super init];
}

- (void)dealloc
{
  // Deallocate backend structure
  free(be_app_reserved);

  [super dealloc];
}

- (void)run
{
  Display *xDisplay;

  [super run];

  // Close the X Display
  xDisplay = [(PXKDPSContext *)[NSDPSContext currentContext] xDisplay];
  XCloseDisplay(xDisplay);
}

@end

//
// GNUstep Backend methods
//
@implementation PXKApplication (GNUstepBackend)

// Get next event
- (NSEvent *)getNextEvent
{
  NSEvent *e;
  e = nextEvent();
  [event_queue enqueueObject: e];
  NSDebugLog(@"Event is for window %d type %d\n", [e windowNumber], [e type]);
  if (e == NullEvent)
    NSDebugLog(@"Got the null event\n");

  return e;
}

// handle a non-translated event
- (void)handleNullEvent
{}

@end

//
// Methods for X Window System implementation
//
@implementation PXKApplication (GNUstepXDPS)

- (Window)xWindow
{
  if (!XWINDOW)
    {
      XGCValues values;
      unsigned long valuemask = (GCForeground | GCBackground);
      int xScreen = [(PXKScreen *)[NSScreen mainScreen] xScreen];
      Display *xDisplay = [(PXKDPSContext *)[NSDPSContext currentContext]
					    xDisplay];

      XWINDOW = RootWindow(xDisplay, xScreen);

      // Create a GC for the content view
      values.foreground = BlackPixel(xDisplay, xScreen);
      values.background = WhitePixel(xDisplay, xScreen);
      XGC = XCreateGC(xDisplay, XWINDOW, valuemask, &values);
    }
  return XWINDOW;
}

- (GC)xGC
{
  return XGC;
}

- (void)setGrabXWindow:(Window)w
{
  XGRAB = w;
}

- (Window)grabXWindow
{
  return XGRAB;
}

@end
