/* 
   PXKButtonCell.m

   NSButtonCell for GNUstep GUI X/DPS Backend

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author:  Scott Christley <scottc@net-community.com>
   Date: March 1996
   
   This file is part of the GNUstep GUI X/DPS Backend.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <gnustep/xdps/PXKButtonCell.h>
#include <gnustep/xdps/PXKView.h>
#include <gnustep/xdps/PXKWindow.h>
#include <gnustep/xdps/PXKDPSContext.h>

@implementation PXKButtonCell

/* Assumes frame has already been translated to X coordinates */
- (void)drawImage:(NSRect)cellFrame
{
#if 0    
    const PXKRect *r = newFrame;
    frame = r[0];

    NSPoint location;
    NSSize size;
    NSImage *image;

    /*
     * Place image centered for now
     */
    location.x = 3;
    location.y = 3;

    image = [self image];
    if (image) {

	size = [image size];

	location.x = cellFrame.size.width /2.0 - size.width/2.0;
	location.y = cellFrame.size.height/2.0 - size.height/2.0;
	
	//[image drawInView:self at:&location];
    }
#endif
}

/* Assumes frame has already been translated to X coordinates */
- (void)drawBorder:(NSRect)cellFrame
{
  // No border then don't draw it
  if ([self isBordered] == NO) {
    return;
  }

  switch (cell_type)
    {
    case NSToggleButton:
    case NSSwitchButton:
    case NSRadioButton:
    case NSMomentaryPushButton:
    case NSPushOnPushOffButton:
      /* Draw border up or down based upon whether highlighted or not */
      if (cell_highlighted)
	[self drawBorderDown: cellFrame];
      else
	[self drawBorderUp: cellFrame];
      break;

    case NSOnOffButton:
    case NSMomentaryChangeButton:
      /* Border is always up */
      [self drawBorderUp: cellFrame];
      break;
    }
}

- (void)drawBorderDown:(NSRect)cellFrame
{
  NSColor *white = [NSColor whiteColor];
  NSColor *black = [NSColor blackColor];
  float bt = 2.25;  // 1/32 of an inch

  // draw the top and left dark edges
  [black set];
  PSnewpath();
  PSmoveto(cellFrame.origin.x, cellFrame.origin.y);
  PSlineto(cellFrame.origin.x,
	   cellFrame.origin.y + cellFrame.size.height);
  PSlineto(cellFrame.origin.x + cellFrame.size.width,
	   cellFrame.origin.y + cellFrame.size.height);
  PSlineto(cellFrame.origin.x + cellFrame.size.width - bt,
	   cellFrame.origin.y + cellFrame.size.height - bt);
  PSlineto(cellFrame.origin.x + bt,
	   cellFrame.origin.y + cellFrame.size.height - bt);
  PSlineto(cellFrame.origin.x + bt, cellFrame.origin.y + bt);
  PSclosepath();
  PSfill();

  // draw the bottom and right light edges
  [white set];
  PSnewpath();
  PSmoveto(cellFrame.origin.x + cellFrame.size.width,
	   cellFrame.origin.y + cellFrame.size.height);
  PSlineto(cellFrame.origin.x + cellFrame.size.width,
	   cellFrame.origin.y);
  PSlineto(cellFrame.origin.x, cellFrame.origin.y);
  PSlineto(cellFrame.origin.x + bt, cellFrame.origin.y + bt);
  PSlineto(cellFrame.origin.x + cellFrame.size.width - bt,
	   cellFrame.origin.y + bt);
  PSlineto(cellFrame.origin.x + cellFrame.size.width - bt,
	   cellFrame.origin.y + cellFrame.size.height - bt);
  PSclosepath();
  PSfill();
}

/* Assumes frame has already been translated to X coordiantes */
- (void)drawBorderUp:(NSRect)cellFrame
{
  NSColor *white = [NSColor whiteColor];
  NSColor *black = [NSColor blackColor];
  float bt = 2.25;  // 1/32 of an inch

  // draw the top and left dark edges
  [white set];
  PSnewpath();
  PSmoveto(cellFrame.origin.x, cellFrame.origin.y);
  PSlineto(cellFrame.origin.x,
	   cellFrame.origin.y + cellFrame.size.height);
  PSlineto(cellFrame.origin.x + cellFrame.size.width,
	   cellFrame.origin.y + cellFrame.size.height);
  PSlineto(cellFrame.origin.x + cellFrame.size.width - bt,
	   cellFrame.origin.y + cellFrame.size.height - bt);
  PSlineto(cellFrame.origin.x + bt,
	   cellFrame.origin.y + cellFrame.size.height - bt);
  PSlineto(cellFrame.origin.x + bt, cellFrame.origin.y + bt);
  PSclosepath();
  PSfill();

  // draw the bottom and right light edges
  [black set];
  PSnewpath();
  PSmoveto(cellFrame.origin.x + cellFrame.size.width,
	   cellFrame.origin.y + cellFrame.size.height);
  PSlineto(cellFrame.origin.x + cellFrame.size.width,
	   cellFrame.origin.y);
  PSlineto(cellFrame.origin.x, cellFrame.origin.y);
  PSlineto(cellFrame.origin.x + bt, cellFrame.origin.y + bt);
  PSlineto(cellFrame.origin.x + cellFrame.size.width - bt,
	   cellFrame.origin.y + bt);
  PSlineto(cellFrame.origin.x + cellFrame.size.width - bt,
	   cellFrame.origin.y + cellFrame.size.height - bt);
  PSclosepath();
  PSfill();
}

/* Assumes frame has already been translated to X coordiantes */
- (void)drawText:(NSRect)cellFrame
{
  float twidth, theight;
  float x, y;
  NSString *title;
  NSColor *black = [NSColor blackColor];
  const char *c;

  title = [self title];
  // No title then nothing to display
  if (!title)
    return;

  // Support must be a font
  if (![support isKindOfClass: [NSFont class]])
    return;

  // Set the font and calculate the string width
  [support set];
  c = [title cString];
  PSstringwidth(c, &twidth, &theight);
  if ([support isKindOfClass: [NSFont class]])
    theight = [support pointSize];
  else
    theight = 0;

  x = (int)(cellFrame.origin.x + ((cellFrame.size.width - twidth)/2.0));
  y = (int)(cellFrame.origin.y + ((cellFrame.size.height - theight)/2.0));
  NSDebugLog(@"PXKButtonCell text %f %f %f %f %f\n", twidth, theight, x, y,
	cellFrame.size.width);

  // Set the color in the current GC
  [black set];
  PSmoveto(x,y);
  PSshow(c);
}

/* Assumes frame has already been translated to X coordiantes */
- (void)drawCheck:(NSRect)cellFrame
{
  NSColor *black = [NSColor blackColor];
  NSColor *white = [NSColor whiteColor];

  float offset = (cellFrame.size.height - 16) / 2;

  if (offset < 0) offset = 0;
#if 0
  // This should really be an image
  if (cell_type == NSRadioButton)
    {
      // Draw a circle
      [black set];
      XDrawArc(xDisplay, xWindow, xGC,
	       cellFrame.origin.x + 3,
	       cellFrame.origin.y + 3 + offset, 10, 10, 0, 360 * 64);
      // Fill the circle based upon cell state
      if (cell_state)
	{
	  [white set];
	  XFillArc(xDisplay, xWindow, xGC,
		   cellFrame.origin.x + 5,
		   cellFrame.origin.y + 5 + offset, 8, 8, 0, 360*64);
	}
    }
  else
    {
      // Draw a square box
      // top and left light edges
      [white set];
      XDrawLine(xDisplay, xWindow, xGC,
		cellFrame.origin.x + 3,
		cellFrame.origin.y + 3 + offset,
		cellFrame.origin.x + 13,
		cellFrame.origin.y + 3 + offset);
      XDrawLine(xDisplay, xWindow, xGC,
		cellFrame.origin.x + 3,
		cellFrame.origin.y + 3 + offset,
		cellFrame.origin.x + 3,
		cellFrame.origin.y + 13 + offset);
      // bottom and right dark edges
      [black set];
      XDrawLine(xDisplay, xWindow, xGC,
		cellFrame.origin.x + 3,
		cellFrame.origin.y + 13 + offset,
		cellFrame.origin.x + 13,
		cellFrame.origin.y + 13 + offset);
      XDrawLine(xDisplay, xWindow, xGC,
		cellFrame.origin.x + 13,
		cellFrame.origin.y + 3 + offset,
		cellFrame.origin.x + 13,
		cellFrame.origin.y + 13 + offset);
      // Draw a check based upon cell state
      if (cell_state)
	{
	  XDrawLine(xDisplay, xWindow, xGC,
		    cellFrame.origin.x + 4,
		    cellFrame.origin.y + 4 + offset,
		    cellFrame.origin.x + 12,
		    cellFrame.origin.y + 12 + offset);
	  XDrawLine(xDisplay, xWindow, xGC,
		    cellFrame.origin.x + 4,
		    cellFrame.origin.y + 12 + offset,
		    cellFrame.origin.x + 12,
		    cellFrame.origin.y + 4 + offset);
	}	  
    }
#endif
}

/* Assumes frame has already been translated to X coordiantes */
- (void)drawFaceLight:(NSRect)cellFrame
{
  NSColor *white = [NSColor whiteColor];

  [white set];
  PSrectfill(cellFrame.origin.x, cellFrame.origin.y,
	     cellFrame.size.width, cellFrame.size.height);
}

/* Assumes frame has already been translated to X coordiantes */
- (void)drawFaceDark:(NSRect)cellFrame
{
  NSColor *gray = [NSColor lightGrayColor];

  [gray set];
  PSrectfill(cellFrame.origin.x, cellFrame.origin.y,
	     cellFrame.size.width, cellFrame.size.height);
}

// OpenStep drawing methods
- (void)drawInteriorWithFrame:(NSRect)cellFrame
		       inView:(NSView *)controlView
{
  NSWindow *wind = [controlView window];
  //  NSRect cf = [controlView convertRect: cellFrame toView: nil];
  NSRect cf = cellFrame;
  BOOL did_lock = NO;

  NSDebugLog(@"PXKButtonCell: drawInteriorWithFrame:inView:\n");

  // If the X/Window hasn't been exposed yet
  // then don't do any drawing
  if (![(PXKWindow *)[control_view window] isXExposed])
    return;

  // Do we need to lock the focus
  if ([NSView focusView] != controlView)
    {
      [controlView lockFocus];
      did_lock = YES;
    }

  // Fill the interior
  switch (cell_type)
    {
    case NSOnOffButton:
    case NSMomentaryPushButton:
      /* Draw button face based upon whether highlighted or not */
      if (cell_highlighted)
	[self drawFaceLight: cf];
      else
	[self drawFaceDark: cf];
      break;

    case NSPushOnPushOffButton:
      /* Draw button face based upon whether its highlighted or not
	 and what its current state is */
      if (cell_state == cell_highlighted)
	[self drawFaceDark: cf];
      else
	[self drawFaceLight: cf];
      break;

    case NSSwitchButton:
    case NSRadioButton:
      // Draw circle or box
      [self drawFaceDark: cf];
      [self drawCheck: cf];
      break;

    case NSMomentaryChangeButton:
    case NSToggleButton:
      /* Button face is always dark */
      [self drawFaceDark: cf];
      break;
    }

  // Draw the image or the text
  if (cell_type == NSImageCellType)
    [self drawImage: cf];
  else
    [self drawText: cf];

  // unlock the focus if we locked it
  if (did_lock)
    [controlView unlockFocus];
}

- (void)drawWithFrame:(NSRect)cellFrame
	       inView:(NSView *)controlView
{
  NSRect cf = [controlView convertRect: cellFrame toView: nil];
  BOOL did_lock = NO;

  // If the X/Window hasn't been exposed yet
  // then don't do any drawing
  if (![(PXKWindow *)[controlView window] isXExposed])
    return;

  // Do we need to lock the focus
  if ([NSView focusView] != controlView)
    {
      [controlView lockFocus];
      did_lock = YES;
    }

  NSDebugLog(@"PXKButtonCell: drawWithFrame:inView:\n");
  NSDebugLog(@"PXKButtonCell: frame %f %f %f %f\n", cellFrame.origin.x,
	     cellFrame.origin.y, cellFrame.size.width, cellFrame.size.height);
  NSDebugLog(@"PXKButtonCell: cframe %f %f %f %f\n", cf.origin.x,
	     cf.origin.y, cf.size.width, cf.size.height);

  // Save last view drawn to
  control_view = controlView;

  // now draw it
  [self drawInteriorWithFrame: cellFrame inView: controlView];
  [self drawBorder:cellFrame];

  // unlock the focus if we locked it
  if (did_lock)
    [controlView unlockFocus];
}

- (void)highlight:(BOOL)lit
	withFrame:(NSRect)cellFrame
	   inView:(NSView *)controlView
{
  [super highlight: lit withFrame: cellFrame inView: controlView];
  [self drawWithFrame: cellFrame inView: controlView];
}

@end
