/* 
   PXKDPSContext.m

   NSDPSContext for GNUstep GUI X/DPS Backend

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author:  Scott Christley <scottc@net-community.com>
   Date: March 1996
   
   This file is part of the GNUstep GUI X/DPS Backend.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <gnustep/xdps/PXKDPSContext.h>
#include <AppKit/NSApplication.h>
#include <gnustep/xdps/PXKWindow.h>
#include <gnustep/xdps/PXKApplication.h>

// Backend instance variables structure
typedef struct _PXKDPSContext_struct
{
  Display *display;
  GC context;
  DPSContext dps_context;
  Window window;
  Window drawable;
} PXKDPSContext_struct;

#define PXKDISPLAY (((PXKDPSContext_struct *)be_context_reserved)->display)
#define PXKGC (((PXKDPSContext_struct *)be_context_reserved)->context)
#define PXKDPS (((PXKDPSContext_struct *)be_context_reserved)->dps_context)
#define PXKWINDOW (((PXKDPSContext_struct *)be_context_reserved)->window)
#define PXKDRAWABLE (((PXKDPSContext_struct *)be_context_reserved)->drawable)

//
// The backend implementation of NSDPSContext
//
@implementation PXKDPSContext

- init
{
  NSMutableData *data = [NSMutableData data];

  return [self initWithMutableData: data
	       forDebugging: NO
	       languageEncoding: dps_ascii
	       nameEncoding: dps_strings
	       textProc: DPSDefaultTextBackstop
	       errorProc: DPSDefaultErrorProc];
}

// Default initializer
- initWithMutableData:(NSMutableData *)data
	 forDebugging:(BOOL)debug
     languageEncoding:(DPSProgramEncoding)langEnc
	 nameEncoding:(DPSNameEncoding)nameEnc
	     textProc:(DPSTextProc)tProc
	    errorProc:(DPSErrorProc)errorProc
{
  [super initWithMutableData: data
	 forDebugging: debug
	 languageEncoding: langEnc
	 nameEncoding: nameEnc
	 textProc: tProc
	 errorProc: errorProc];

  // Allocate backend structure
  be_context_reserved = malloc(sizeof(PXKDPSContext_struct));

  PXKDISPLAY = NULL;
  PXKWINDOW = 0;
  PXKDPS = NULL;
  PXKDRAWABLE = 0;

  return self;
}

- (void)dealloc
{
  // Destroy the context
  DPSDestroySpace(DPSSpaceFromContext(PXKDPS));

  // Release backend structure
  free(be_context_reserved);

  [super dealloc];
}

@end

//
// Methods for XWindows implementation
//
@implementation PXKDPSContext (GNUstepXDPS)

- (Display *)xDisplay
{
  return PXKDISPLAY;
}

- (void)setXDisplay:(Display *)xdisplay
{
  PXKDISPLAY = xdisplay;
}

- (GC)xGC
{
  return PXKGC;
}

/* return the current GC because the caller may want to restore */
- (GC)setGC:(GC)xgc
{
  GC temp = PXKGC;
  PXKGC = xgc;
  return temp;
}

- (DPSContext)xDPSContext
{
  return PXKDPS;
}

- (void)createDPSContext
{
  NSApplication *theApp = [NSApplication sharedApplication];
  GC xGC;

  PXKWINDOW = [(PXKApplication *)theApp xWindow];
  xGC = [(PXKApplication *)theApp xGC];

  // Create the context if need be
  if (!PXKDPS)
    {
      NSLog(@"DPS xWindow %x\n", PXKWINDOW);
      PXKDPS = XDPSCreateSimpleContext(PXKDISPLAY, PXKWINDOW, xGC, 0, 0,
				       DPSDefaultTextBackstop,
				       DPSDefaultErrorProc, NULL);
      if (PXKDPS == NULL)
	{
	  NSLog(@"DPS is not available\n");
	  exit(1);
	}
      // Make it the active context
      DPSSetContext(PXKDPS);
    }
}

- (void)setXDrawable:(Window)w
{
  PXKDRAWABLE = w;
}

- (Window)xDrawable
{
  return PXKDRAWABLE;
}

@end
