/* 
   PXKImage.m

   NSImage for GNUstep GUI X/DPS Backend.

   Copyright (C) 1996 Free Software Foundation, Inc.

   Written by:  Adam Fedor <fedor@colorado.edu>
   Date: Feb 1996
   
   This file is part of the GNUstep GUI X/DPS Backend.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <gnustep/xdps/PXKImage.h>
#include <gnustep/xdps/PXKView.h>
#include <gnustep/xdps/PXKScreen.h>
#include <gnustep/xdps/PXKDPSContext.h>
#include <gnustep/xdps/PXKWindow.h>
#include <gnustep/xdps/PXKCachedImageRep.h>

@implementation PXKImage

- (void) _displayEraseRect: (NSRect)bounds view: (NSView *)view
        color: (NSColor *)color
{
  if (color)
    [color set];
  else
    [[NSColor whiteColor] set];
  //PSsetgray(NX_WHITE);

  NSRectFill(bounds);
}

- (void) compositeToPoint: (NSPoint)point fromRect: (NSRect)rect
	operation: (NSCompositingOperation)op
{
  NSCachedImageRep* rep;

  /* Create and draw to a cache, if needed */
  [super compositeToPoint: point fromRect: rect operation: op];

  /* Get the cache and composite it to our lock focused view */
  rep = [self bestRepresentationForDevice: nil];
  //  PScomposite(NSMinX(rect), NSMinY(rect), NSWidth(rect), NSHeight(rect),
  //	      [[rep window] gState], point.x, point.y, op);

}

- (void) dissolveToPoint: (NSPoint)point fromRect: (NSRect)rect
        fraction: (float)aFloat
{
  NSCachedImageRep* rep;

  /* Create and draw to a cache, if needed */
  [super dissolveToPoint: point fromRect: rect fraction: aFloat];

  /* Get the cache and dissolve it to our lock focused view */
  rep = [self bestRepresentationForDevice: nil];

}

@end



//
// old stuff
//
#if 0
@implementation PXKImage

- drawInView:(NSView *)view at:(const NSPoint *)location;
{
  NSDPSContext *cur = [NSDPSContext currentContext];
  Window xWindow = [(PXKWindow *)[view window]xWindow];
  Display *xDisplay = [(PXKDPSContext *)cur xDisplay];
  NSColor *black = [NSColor blackColor];
  GC xGC;
  const NSPoint *p = location;

  [black set];
  xGC = [(PXKDPSContext *)cur xGC];
  XPutImage(xDisplay, xWindow, xGC, [self xImage],
	    0,0, p[0].x, p[0].y, image_size.width, image_size.height);

  return self;
}

- buildImageFromString:(const char *)image;
{
#if 0
    char buf[15];
    int  version;
    int  width;
    int  height;
    int  x, y;
    char pixel;
    XColor c1, c2;
    unsigned int white = WhitePixel(pxkDisplay, [(PXKScreen *)
						    [NSScreen mainScreen]
						    xScreen]);
    unsigned int lightgray;
    unsigned int darkgray;
    unsigned int black = BlackPixel(pxkDisplay, [(PXKScreen *)[NSScreen mainScreen] xScreen]);

    strncpy(buf, image+12, 4);
    width = atoi(buf);

    strncpy(buf, image+16, 4);
    height = atoi(buf);

    NSDebugLog(@"width: %d  height: %d\n", width, height);
    
    XAllocNamedColor(pxkDisplay,
		     XDefaultColormap(pxkDisplay, 
				      [(PXKScreen *)[NSScreen mainScreen] xScreen]),
		     "light gray", &c1, &c2);

    lightgray = c1.pixel;
    
    XAllocNamedColor(pxkDisplay,
		     XDefaultColormap(pxkDisplay, [(PXKScreen *)[NSScreen mainScreen] xScreen]),
		     "dark gray", &c1, &c2);

    darkgray = c1.pixel;

    for (y=0; y<height; y++) {
	for (x=0; x<width; x++) {

	    pixel = image[20 + x + y*width]; 

	    NSDebugLog(@"%c", pixel);

	    switch(pixel) {
	      case 'g': XPutPixel([self xImage], x, y, lightgray); break;
	      case 'b': XPutPixel([self xImage], x, y, black); break;
	      case 'G': XPutPixel([self xImage], x, y, darkgray); break;
	      case 'w': XPutPixel([self xImage], x, y, white); break;
	      default: break;
	    }
	}

	NSDebugLog(@"\n");

    }
#endif
    return self;
}
    
- initWithSize:(const NSSize *)initialSize;
{
#if 0
    int x, y;
    XColor c1, c2;
    const NSSize *si = initialSize;
    XImage *xImage;

    [super init];
    image_size = si[0];

    NSDebugLog(@"NSImage: initWithSize:");
    
    /*
     * This is the only way I found to reliably create an x image.
     */    
    xImage = XGetImage(pxkDisplay, RootWindow(pxkDisplay, [(PXKScreen *)[NSScreen mainScreen] xScreen]),
		       0,0, image_size.width, image_size.height, 
		       AllPlanes, XYPixmap);

    /*
     * Fill the image with the default color, lightgray
     */
    XAllocNamedColor(pxkDisplay,
		     XDefaultColormap(pxkDisplay, [(PXKScreen *)[NSScreen mainScreen] xScreen]),
		     "light gray", &c1, &c2);

    for (x=0; x<image_size.width; x++) {
	for (y=0; y<image_size.height; y++) {
	    XPutPixel([self xImage], x, y, c1.pixel);
	}
    }
    
#if 0
    XPutPixel(xImage, 5,5, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 5,6, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 5,7, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 5,8, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 6,5, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 6,6, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 6,7, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 6,8, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 7,5, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 7,6, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 7,7, BlackPixel(NSDisplay, NSScreen));
    XPutPixel(xImage, 7,8, BlackPixel(NSDisplay, NSScreen));
#endif
#endif    
    return self;
}

- initFromBitmap:(char *)bits size:(const NSSize *)newSize;
{
    const NSSize *si = newSize;

    [super init];
    image_size = si[0];
  
    /* See X book number 1, page 157-158 */    

    NSDebugLog(@"w: %f h:%f\n", image_size.width, image_size.height);
#if 0
    xPixmap = XCreateBitmapFromData(NSDisplay,
				    RootWindow(NSDisplay, NSScreen),
				    _NSRightArrow_bitmap,
				    15, 15);
#endif
    
    return self;
}
    
- initFromPath:(const char *)filename size:(const NSSize *)newSize;
{
  return self;
}

- (XImage *)xImage;
{
  //return xImage;
  return NULL;
}

@end
#endif
