/* 
   PXKTextField.m

   NSTextField for GNUstep GUI X/DPS Backend.

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author:  Pascal Forget <pascal@wsc.com>
   Date: March 1996
   
   This file is part of the GNUstep GUI X/DPS Backend.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <gnustep/xdps/PXKTextField.h>
#include <gnustep/xdps/PXKView.h>
#include <gnustep/xdps/PXKApplication.h>

@implementation PXKTextField

- (void)selectText:sender
{
  [super selectText: sender];
  [self display];
}
			       
/*
 * Get characters until you encounter
 * a carriage return, return number of characters.
 * Deal with backspaces, etc.  Deal with Expose events
 * on all windows associated with this application.
 * Deal with keyboard remapping.
 */

- (void)keyDown:(NSEvent *)theEvent;
{
  NSMutableString *s;
  NSString *c;
  unsigned int flags = [theEvent modifierFlags];
  unsigned int key_code = [theEvent keyCode];

  NSDebugLog(@"PXKTextField: -keyDown %s\n", [[theEvent characters] cString]);

  // If SHIFT-TAB key then make the previous text the first responder
  if ((key_code == 0x09) && (flags & NSShiftKeyMask))
    {
      if ([previous_text respondsToSelector:@selector(selectText:)])
	// Either select the previous' text
	[previous_text selectText:self];
      else
	// Or select ourself
	[self selectText:self];

      // Have the target perform the action
      [self sendAction:[self action] to:[self target]];

      return;
    }

  // If TAB key then make the next text the first responder
  if (key_code == 0x09)
    {
      if ([next_text respondsToSelector:@selector(selectText:)])
	// Either select the next's text
	[next_text selectText:self];
      else
	// Or select ourself
	[self selectText:self];

      // Have the target perform the action
      [self sendAction:[self action] to:[self target]];

      return;
    }

  // If RETURN key then make the next text the first responder
  if (key_code == 0x0d)
    {
      if ([next_text respondsToSelector:@selector(selectText:)])
	// Either select the next's text
	[next_text selectText:self];
      else
	// Or select ourself
	[self selectText:self];

      // Have the target perform the action
      [self sendAction:[self action] to:[self target]];

      return;
    }

  // If not editable then don't recognize the key down
  if (![self isEditable]) return;

  // Get a mutable copy of the contents
  c = [self stringValue];
  s = @"";

  // If BACKSPACE key then delete the last character
  if (key_code == NSDeleteFunctionKey)
    {
      unsigned int l = [c cStringLength];
      char *nc, *cc;

      // If string is empty then nothing to delete
      if (l == 0) return;

      // remove last character
      cc = [c cString];
      nc = (char *)malloc(l+1);
      strcpy(nc, cc);
      nc[l-1] = '\0';
      s = [NSMutableString stringWithCString: nc];
      free(nc);
    }
  else
    {
      // Add the characters
      s = [[NSMutableString alloc] initWithString: c];
      [s appendString: [theEvent characters]];
    }

  // Set our new value -this should force a redisplay
  [self setStringValue: s];
}

- (void)mouseDown:(NSEvent *)theEvent;
{
  [self selectText: self];
}

@end
