/* 
   PXKView.m

   NSView for GNUstep GUI X/DPS Backend.

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author:  Pascal Forget <pascal@wsc.com>
   Date: January 1996
   
   This file is part of the GNUstep GUI X/DPS Backend.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include <gnustep/xdps/PXKView.h>
#include <gnustep/xdps/PXKDPSContext.h>
#include <gnustep/xdps/PXKWindow.h>

@implementation PXKView

- (void)lockFocus
{
  NSDPSContext *cur = [NSDPSContext currentContext];
  NSWindow *w = [self window];

  [super lockFocus];

  // The window must be exposed/mapped
  if (![(PXKWindow *)w isXExposed])
    return;

  // Set the Xgcdrawable if we are the content view
  if (self == [w contentView])
    {
      float x, y;
      Window xWindow = [(PXKWindow *)w xWindow];

      // Convert view size to device coordinates
      PSdtransform(bounds.size.width, bounds.size.height, &x, &y);
      if (y < 0) y = -y;

      PSsetXgcdrawable(XGContextFromGC([(PXKWindow *)w xGC]), 
		       xWindow, 0, y);

      [(PXKDPSContext *)cur setXDrawable: xWindow];
    }

  // Save the graphics state
  PSgsave();

  //
  // Apply the view transformations
  //

  // Translate to the view frame
  PStranslate(frame.origin.x, frame.origin.y);
}

- (void)unlockFocus
{
  NSWindow *w = [self window];

  [super unlockFocus];

  // The window must be exposed/mapped
  if (![(PXKWindow *)w isXExposed])
    return;

  // Restore the graphics state
  PSgrestore();
}

@end
