/* 
   TestView.h

   Simple subclass of NSView.

   Copyright (C) 1996 Free Software Foundation, Inc.

   Author:  Scott Christley <scottc@net-community.com>
   Date: June 1996
   
   This file is part of the GNUstep GUI X/DPS Backend.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   If you are interested in a warranty or support for this source code,
   contact Scott Christley <scottc@net-community.com> for more information.
   
   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/ 

#include "TestView.h"
#include <gnustep/xdps/PXKWindow.h>
#include <gnustep/xdps/PXKScreen.h>
#include <gnustep/xdps/PXKDPSContext.h>
#include <gnustep/xdps/PXKColor.h>
#include <gnustep/gui/NSFont.h>
#include <gnustep/xdps/PXKMenu.h>

@implementation TestView

// Class methods

// Instance methods
- (void)drawRect:(NSRect)rect
{
  NSFont *f;
  float width, height;
  NSColor *c = [NSColor greenColor];
  NSColor *blue = [NSColor blueColor];
  NSColor *y = [NSColor yellowColor];
  NSColor *mg = [NSColor magentaColor];
  NSColor *orange = [NSColor orangeColor];

  NSLog(@"Painting TestView %f %f %f %f\n", bounds.origin.x, bounds.origin.y,
	bounds.size.width, bounds.size.height);

  [orange set];
  NSRectFill(bounds);

  [c set];

#if 1
  // Text
  f = [NSFont boldSystemFontOfSize: 24];
  [f set];
  PSmoveto(10, 10);
  PSshow("Bold system font.");
  PSstringwidth("Bold system font.", &width, &height);
  NSLog(@"width: %f height: %f\n", width, height);

  f = [NSFont systemFontOfSize: 24];
  [f set];
  PSmoveto(10, 50);
  PSshow("System font.");
  PSstringwidth("System font.", &width, &height);
  NSLog(@"width: %f height: %f\n", width, height);

  f = [NSFont userFixedPitchFontOfSize: 24];
  [f set];
  PSmoveto(10, 100);
  PSshow("User fixed pitch font.");
  PSstringwidth("User fixed pitch font.", &width, &height);
  NSLog(@"width: %f height: %f\n", width, height);

  f = [NSFont userFontOfSize: 24];
  [f set];
  PSmoveto(10, 150);
  PSshow("User font.");
  PSstringwidth("User font.", &width, &height);
  NSLog(@"width: %f height: %f\n", width, height);
#endif

#if 1
  // Absolute Lines
  [blue set];
  PSnewpath();
  PSmoveto(400, 400);
  PSlineto(420, 400);
  PSlineto(440, 380);
  PSlineto(440, 360);
  PSlineto(420, 340);
  PSlineto(400, 340);
  PSlineto(380, 360);
  PSlineto(380, 380);
  PSclosepath();
  PSfill();
#endif

#if 1
  // Relative Lines
  [y set];
  PSnewpath();
  PSmoveto(400, 200);
  PSrlineto(20, 0);
  PSrlineto(20, -20);
  PSrlineto(0, -20);
  PSrlineto(-20, -20);
  PSrlineto(-20, 0);
  PSrlineto(-20, 20);
  PSrlineto(0, 20);
  PSclosepath();
  PSstroke();
#endif

#if 1
  // Arcs
  [mg set];
  PSnewpath();
  PSarc(200, 200, 100, 0, 90);
  PSarcn(200, 200, 50, 90, 0);
  PSclosepath();
  PSfill();
#endif

#if 1
  // Absolute Beziers
  PSnewpath();
  PSmoveto(200, 400);
  PScurveto(250, 450, 280, 800, 180, 780);
  PScurveto(250, 600, 100, 500, 200, 400);
  PSclosepath();
  PSfill();
#endif

#if 1
  // Relative Beziers
  PSnewpath();
  PSmoveto(500, 400);
  PSrcurveto(50, 50, 80, 400, -20, 380);
  PSrcurveto(70, -180, -80, -280, 20, -380);
  PSclosepath();
  PSstroke();
#endif
}

- (void)mouseDown:(NSEvent *)theEvent
{
#if 0
  NSApplication *theApp = [NSApplication sharedApplication];
  NSMenu *m = [theApp mainMenu], *sm;
  NSArray *mi = [m itemArray];
  NSMenuCell *mc;
  id e;
  NSRect r;

  e = [mi objectEnumerator];
  mc = (NSMenuCell *)[e nextObject];
  while (mc)
    {
      if ([mc hasSubmenu])
	{
	  sm = [mc submenu];
	  break;
	}
      mc = (NSMenuCell *)[e nextObject];
    }

  NSLog(@"Got submenu %s\n", [[sm title] cString]);
  [(PXKMenu *)sm createXWindow: r];
#endif
}

@end
