\# $Id: pthread_blockasync_np.man,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_blockasync_np.man,v $
\# Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
\# PCThreads for Linux.
\#
\# Revision 1.1.1.1  1996/06/29 01:20:50  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:47  mtp
\# + Release 2.0
\#
\# Revision 1.1  1996/01/14 20:10:49  mtp
\# + Placed under CVS control.
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_blockasync_np \- Block asynchronous context switches
.br
pthread_unblockasync_np \- Unblock asynchronous context switches
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "pthread_blockasync_np("void);
.br
.B void
.BI "pthread_unblockasync_np(int " flag );
.SH DESCRIPTION
These two functions are used to block and unblock asynchronous thread context
switching.  When called, the
.B pthread_blockasync_np()
function informs the threads runtime not to preempt the calling thread during
a clock interrupt.  All other timer-related functions continue, however.  To
reestablish asynchronous context switching, the
.B pthread_unblockasync_np()
function is called.
.PP
The
.B pthread_blockasync_np()
function atomically sets the asynchronous preemption flag in the thread's
runtime and returns its previous state. The
.B pthread_unblockasync_np()
function atomically sets (i.e., restores) the asynchronous preemption flag to
the value of the
.BI flag
argument.
.PP
While asynchronous context switching is blocked, services that may lead to
synchronous preemption (e.g.,
.B pthread_mutex_block(), pthread_delay_np(), pthread_setprio_np()
) should not be called.
.PP
These two services are provided for [legacy] programs that must integrate
threads with existing interrupt handlers.  They provide a way to guarantee that
the thread, while executing in an interrupt handler, will not be preempted.
.SH PARAMETERS
.SS Input Parameters
.IP flag
An integer valued variable containing the asynchronous preemption state returned
by a previous call to
.B pthread_blockasync_np().
.SS Return Parameters
.IP None
.SH RETURN VALUE
For
.B pthread_blockasync_np()
the return value is the previous value of the asynchronous preemption flag.
This routine does not return an error value and can not fail.  The
.B pthread_unblock_async_np()
does not return a value.
.SH USAGE EXAMPLE
.PP
Block asynchronous preemption during execution of a signal handler.
.PP
.nf
        /*
         * This might be a signal handler installed 
         * by sigaction(), for example.
         */
        void
        signal_handler( int signum )
        {
            int flag;

            flag = pthread_blockasync_np();

            <...>

            pthread_unblockasync_np( flag );
        }
.fi
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B sigaction(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec.com,  mtp@big.aa.net>







