\# $Id: pthread_cancel.man,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_cancel.man,v $
\# Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
\# PCThreads for Linux.
\#
\# Revision 1.1.1.1  1996/06/29 01:20:50  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:48  mtp
\# + Release 2.0
\#
\# Revision 1.4  1996/01/14 20:22:23  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.3  1996/01/14 18:32:05  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.2  1996/01/10 19:03:22  mtp
\# + Fixed mispellings and clumsy wording.
\#
\# Revision 1.1  1996/01/10 17:17:06  mtp
\# + Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_cancel \- Request that a thread be terminated
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "pthread_cancel(pthread_t " thread )
.SH DESCRIPTION
The
.B pthread_cancel()
function allows one thread to request the termination of itself, or another
thread. A thread may dictate whether it is cancelable by setting its cancel
state to either either PTHREAD_CANCEL_ENABLE or PTHREAD_CANCEL_DISABLE (A
thread sets its cancel state with the function
.B pthread_setcancelstate()
).  If a thread's cancel state is PTHREAD_CANCEL_ENABLE, it will terminate
itself as soon as it calls a function that has been defined as a cancellation
point.   The POSIX .1c standard specifies the following pthread functions as
cancellation points:
.sp
.RS
.BI pthread_cond_wait()
.br
.BI pthread_cond_timedwait()
.br
.BI sigwait()
.br
.BI pthread_join()
.br
.BI pthread_testcancel()
.RE
.sp
In addition, the
.B pthread_delay_np()
function in PCthreads is also a cancellation point (to maintain compatibility
with DCE threads - Draft 4 of the POSIX .1c API).
.sp
Termination processing when a cancel is delivered is as if the cancelled
thread called
.B pthread_exit()
with an exit status value of PTHREAD_CANCELED.  To this end, when a thread
executes a cancel, all outstanding cleanup routines are first executed, then any
thread-specific destructors are called, and finally, any outstanding join
requests are satisfied.

.SH PARAMETERS
.SS Input Parameters
.IP thread
The identifier of the thread to which the cancel is to be delivered.  This value
may not be NULL.
.SS Return Parameters
.IP None
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
The value specified by the thread parameter is NULL or references a non-thread
object.
.TP
.IP ENOMEM
The process's heap has become exhausted.
.TP
.IP ESRCH
The thread identifier does not specify an existing thread.
.SH USAGE EXAMPLE
.PP
The following code fragment illustrates how to cancel a thread.
.PP
.nf
        void wait_for_msg( int *fd )
        {
            /*
             * Make sure cancels are enabled and asynchronous
             * cancels are disabled.
             */
            pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &prev_state);
            pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, &prev_type);

            /*
             * Wait for a message.  Periodically wake up and
             * test whether we've been requested to cancel.
             */
            while( !wait_on_sockfd( fd, &interval ) )
                pthread_testcancel();
        }

        main()
        {

            /*
             * Pass a socket file descriptor to the waiter
             * thread.
             */
        pthread_create( &th1, NULL, wait_for_msg, &fd );
                .
                .
                .
        pthread_cancel( th1 );
                .
                .
        }
.fi
Presumably, the job of th1 is to wait on a socket for a message to appear.
Periodically, it checks whether another thread has posted a cancel requesting it
to cease waiting and terminate.  This is a principal use of cancellation,
terminating a thread in an unbounded wait.
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_setcancelstate(), pthread_setcanceltype(), pthread_testcancel(),
.B pthread_cleanup_pop(), pthread_cleanup_push()
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec.com,  mtp@big.aa.net>







