\# $Id: pthread_cond_signal.man,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_cond_signal.man,v $
\# Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
\# PCThreads for Linux.
\#
\# Revision 1.1.1.1  1996/06/29 01:20:50  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:48  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:24  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:05  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_cond_signal \- Wake a single thread at a condition variable
.sp
pthread_cond_broadcast \- Wake all threads at a condition variable
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "pthread_cond_signal(pthread_cond_t *" cond )
.sp
.B int
.br
.BI "pthread_cond_broadcast(pthread_cond_t *" cond )
.SH DESCRIPTION
The
.B pthread_cond_signal()
function causes at most one thread to be unblocked at the condition variable
specified by the
.I cond
parameter.  The
.B pthread_cond_broadcast()
is similar in function except that all threads waiting for the specified
condition variable are unblocked and allowed to proceed.  If no threads are
waiting for the specified condition variable to be signaled, both functions
return harmlessly.

.PP
When signaled, the thread selected to return is always the thread with the
highest priority waiting at the specified condition variable.  If multiple
threads of identical priorities are waiting, the thread that has been
waiting the longest is unblocked.  When broadcast, the threads are unblocked
in priority order, highest to lowest.
.SH PARAMETERS
.SS Input Parameters
.IP cond
The address of a variable containing the identifier of the condition variable
to be signaled or broadcast.
.SS Return Parameters
.IP None
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
The value specified by
.I cond
is invalid (i.e., not a condition variable handle) or the parameter itself
is NULL.
.TP
.IP ENOMEM
The process's heap has become exhausted.
.SH USAGE EXAMPLE
.PP
One line description of example
.PP
Signal a thread at a condtion variable.
.nf
                    .
                    .
                    .
        status = pthread_mutex_lock( &cond_mu );
        CHECK(status, "Lock failed");

        data_is_ready = TRUE;

        status = pthread_cond_signal( &cond );
        CHECK(status, "Condition signal failed");

        status = pthread_mutex_unlock( &cond_mu );
        CHECK(status, "Lock failed");
                    .
                    .
                    .
.fi
Note that maintaining the mutex lock across the call to
.B pthread_cond_signal()
is not required.  In this case, the mutex
.I cond_mu
is not unlocked until it returns from
.B pthread_cond_signal().
This illustrates how a signaling thread can preclude preemption by higher
priority waiter threads.
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_cond_wait(3), pthread_cond_timedwait(3), pthread_mutex_lock(3)
.B pthread_mutex_unlock(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>
