\# $Id: pthread_cond_wait.man,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_cond_wait.man,v $
\# Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
\# PCThreads for Linux.
\#
\# Revision 1.1.1.1  1996/06/29 01:20:50  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:49  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:24  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:06  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_cond_wait \- Wait for a condition variable to be signaled.
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "pthread_cond_wait(pthread_cond_t *" cond ","
.br
.BI "                  pthread_mutex_t *" mutex )
.SH DESCRIPTION
The
.B pthread_cond_wait()
routine causes a thread to wait until a condition variable becomes signaled (or
broadcast).  A condition variable is an abstraction corresponding to one or more
boolean relations (aka predicates) governing the state of some shared data.  The
thread calling this service waits (is blocked!) until a separate thread,
presumably monitoring the state of the shared data, sets the data's predicate to
TRUE and then signals the condition variable.  The condition variable, specified
by the
.I cond
parameter, is signaled by calling the
.B pthread_cond_signal()
or
.B pthread_cond_broadcast()
function.
.PP
To call the
.B pthread_cond_wait()
routine successfully requires that the calling thread hold the
mutex (as specified by the
.I mutex
parameter) in a locked state.  If called with this mutex unlocked, an error
value of ENOLCK is returned.  When entered, the
.B pthread_cond_wait()
function atomically releases the mutex and causes the calling thread to wait
for condition variable to be signaled.  When the wait is satisfied, the mutex is
relocked and the function returns.  Threads continue to wait at condition
variables until the condition is signaled.
.PP
On some systems, notably those with symmetrical multiprocessor support, a
low probability exists that a thread may return from this routine without
the condition variable having been signaled (when this occurs, the mutex
is reacquired before the routine returns).  To handle these situations,
good thread-programming practice dictates that the
.B pthread_cond_wait()
service be enclosed in a loop that precludes a thread from accessing the
shared data unless returning from a
.I signaled
condition variable. When guarded properly, if such
.I spurious
returns occur, the errant thread will reenter the condition wait because the
predicate will not have changed.
.SH PARAMETERS
.SS Input Parameters
.IP cond
The address containing the identifier of the condition variable at which the
calling thread is to wait.  This value may not be NULL.
.IP mutex
The address containing the identifier of the mutex guarding the data predicate.
When 
.B pthread_cond_wait()
is called, the mutex is unlocked and made available to other threads.  When
the condition variable is signaled, the mutex is relocked by the
thread returning from the
.B pthread_cond_wait()
function.
.SS Return Parameters
.IP None
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
The value specified by one or more of the parameters is invalid, or the
parameter itself was NULL.
.TP
.IP ENOMEM
The process's heap has become exhausted.
.TP
.IP ENOLCK
The mutex specified by
.I mutex
is not currently locked by the calling thread.
.SH USAGE EXAMPLE
.PP
Lock a mutex and then call 
.B pthread_cond_wait().
.nf
                .
                .
                .
    /*
     * Synchronize child threads with their parent using a condition 
     * variable, of which the predicate (thread_hold) will be controlled
     * by the parent.
     */
    status = pthread_mutex_lock(&cond_mutex);
    CHECK(status, "Failed to lock mutex");

    while( thread_hold )
    {
        status = pthread_cond_wait(&cond_var, &cond_mutex);
        check(status, "2:Cond_wait bad status\n");
    }
                .
                .
            < Access the shared data >
                .
                .
    status = pthread_mutex_unlock(&cond_mutex);
    CHECK(status, "Failed to unlock mutex");

.fi
.PP
Note that the mutex,
.I cond_mutex,
is locked prior to calling
.B pthread_cond_wait().
Because the mutex is reacquired prior to returning from the
.B pthread_cond_wait()
function, the thread may safely access the data guarded by the
predicate
.I thread_hold
boolean.
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_mutex_lock(3), pthread_mutex_unlock(3),
.B pthread_cond_signal(3), pthread_cond_broadcast(3),
.B pthread_cond_timedwait(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>







