\# $Id: pthread_create.man,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_create.man,v $
\# Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
\# PCThreads for Linux.
\#
\# Revision 1.1.1.1  1996/06/29 01:20:50  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:49  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:25  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:06  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_create \- Create a new thread of execution
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br 
.BI "pthread_create(pthread_t *" thread ","
.br
.BI "               pthread_attr_t *" attr ","
.br
.BI "               (void *) (*" thread_function ")(void *" arg ")" ","
.br
.BI "               void *" thread_arg )
.SH DESCRIPTION
The
.B pthread_create()
function creates a single thread of execution within a process's application.
The thread executes the thread function, 
.B thread_function,
independent of all other threads.
.PP
Once created, new threads are placed on a ready queue where they wait to be
scheduled. The new thread will begin execution when the priority and policy with
which it was created dictates.  In general, the highest priority thread always
executes before lower priority threads.  For example, if the new thread is
created with a higher priority than that of the thread calling

.B pthread_create(),
the new thread will preempt the creating thread and begin its execution.  This
means that under these circumstances, the new thread may complete before the
creating thread returns from the call to
.B pthread_create().
.PP
The signal state of the new thread is initialized by inheriting the signal mask
from the creating thread.  The new thread's set of pending signals is clear.
.SH PARAMETERS
.SS Return Parameters
.IP thread
A pointer to the identifier of the newly created thread.  Under POSIX rules,
new threads are created with
.B PTHREAD_CREATE_DETACHED
semantics.  This permits the runtime library to recover the resources
allocated to threads without the programmer having to explicitly arrange to do
so (see
.B pthread_detach()).
However, under some circumstances, the new thread may have already completed by 
the time the
.B pthread_create()
function returns.  Thus, this parameter may reference a non-existant thread.
.SS Input Parameters
.IP attr
The address of a handle to a thread attribute's object.  PCthreads
allows threads to be created with the following attributes:
.RS
.TP
.B \(bu
.I StackSize
specifies the number of bytes to allocate to a thread's stack.  Once created,
a thread's stack may not be modified.  If not specified, PCthreads are 
created with 8196 bytes of stack.  The POSIX function,
.B pthread_attr_setstacksize() is used to set the stack size of a thread
attributes object.
.TP
.B \(bu
.I ExecutionPriority
specifies the execution priority with which the thread is initially created.
If this priority is greater than that of the creating thread, the new thread
will begin its execution before the creating thread returns from
.B .pthread_create().
If not specified, PCthreads are created with a scheduling priority of
.PRI_FIFO_MIN.
.TP
.B \(bu
.I SchedulingPolicy
specifies the policy to be used by the runtime library to determine when the
thread is to run.  The PCthreads package currently supports three
scheduling policies,
.B SCHED_FIFO
(first-in, first-out), 
.B SCHED_RR
(round-robin),
and
.B SCHED_OTHER
(priority decay). For a detailed description of this policy, please see the
man page for
.B pthread_attr_setschedpolicy()
.TP
.B \(bu
.I SchedulingQuantum
specifies the number of time slices allocated to a thread before becoming
eligible for isosynchronous preemption, i.e., preempted during the execution of
the timer interrupt. If not specified, the default quantum is two time slices.
In Linux, the thread timer interrupts an executing thread once every N
milliseconds.  The default value for N is 20 milliseconds but can be changed by
modifying the .pthread_initrc file (see .B pthread_info man page).  Applications
should use this scheduling parameter with great reluctance, since POSIX does not
recognize this parameter and its use is non-portable.
.TP
.B \(bu
.I InheritSchedule
specifies under what circumstances the creating thread's scheduling priority,
quantum, and policy are passed on (i.e., inherited by) a new thread.  See the
man page for
.B pthread_attr_setinheritsched()
for a more detailed description.
.TP
.B \(bu
.I DetachState
specifies whether the thread is to be created as 
.B detached
(the default) or
.B joinable.
Unless specified otherwise, new threads are created detached. See the man page
for
.B pthread_attr_setdetachstate()
for more information.
.RE
.IP thread_function
A pointer to the procedure the thread is to execute.
.IP thread_arg
A pointer to the argument of the thread's procedure.  This parameter is passed
to the thread's procedure.
.SH RETURN VALUE
A value of zero (0 or SUCCESS) is returned if no errors are encountered.
When an error condition arises, no thread is created, the thread referenced by
the
.B thread
parameter is undefined, and this routine returns an integer value indicating
the reason for the error.  Possible values are:
.TP
.IP EAGAIN
The system lacks the necessary resources to create the new thread.
.TP
.IP EINVAL
The value specified by one or more of the parameters is invalid.
.TP
.IP EPERM
The calling thread does not have the appropriate permission to create a thread
with the specified attributes.  Currently, permissions are unenforced in Linux
Threads.
.SH USAGE EXAMPLE
.PP
Create a thread with a stack of 4096 bytes:
.nf

#include <stdlib.h>
#include <pthread.h>

extern void *proc( void *arg );   /* Function new thread is to execute */
#define STACK_SIZE ((size_t) 4096)

pthread_attr_t th_attr;
pthread_t thread;

int
main( int argc, char *argv[] )
{
    int exit_status = SUCCESS;

	/*
	 * Initialize an attributes object handle, then pass a pointer to
	 * to the handle into the pthread_create() routine.
	 */
	(void) pthread_attr_init( &th_attr );
	(void) pthread_attr_setstacksize( &th_attr, STACK_SIZE );

	(void) pthread_create( &thread, &th_attr, proc, NULL );

	/*
	 * Wait for the new thread to complete its execution.  The 
	 * exit_status parameter is used to retrieve the new thread's
     * exit status, if any.
	 */
	(void) pthread_join( thread, (void **) &exit_status );
	(void) pthread_attr_destroy( &th_attr );
	return( EXIT_SUCCESS );
}
.fi
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_attr_init(3)
.B pthread_attr_destroy(3)
.B pthread_attr_setstacksize(3)
.B pthread_attr_setschedparam(3)
.B pthread_join(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>
