\# $Id: pthread_mutex_init.man,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_mutex_init.man,v $
\# Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
\# PCThreads for Linux.
\#
\# Revision 1.1.1.1  1996/06/29 01:20:51  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:51  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:27  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:08  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_mutex_init \- Initialize a mutex object
.br
pthread_mutex_destroy \- Delete a mutex object 
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "pthread_mutex_init(pthread_mutex_t *" mutex ","
.br
.BI "                   pthread_mutexattr_t *" attr )
.sp
.B int
.br
.BI "pthread_mutex_destroy(pthread_mutex_t *" mutex )
.SH DESCRIPTION
The
.B pthread_mutex_init()
function initializes a mutex object.  Mutex objects 
are used by the calling thread to serialize access to shared data (i.e.,
variables visible to two or more threads). The
.B pthread_mutex_destroy()
function is used to delete an existing mutex and recover the mutex's resources.
.SH PARAMETERS
.SS Input Parameters
.IP mutex
This parameter is used as an input parameter to the
.B pthread_mutex_destroy()
function.  It is the address of memory containing the identifier of a mutex to
be destroyed.
.IP attr
Used by the
.B pthread_mutex_init()
function, it is the address of memory containing the identifier of the mutex
attributes object used by the thread's runtime to configure a new mutex.  If the
value of this parameter is NULL, the mutex will be initialized with the system's
default attribute values.
.SS Return Parameters
.IP mutex
The address of memory into which the identifier of the newly created mutex
will be stored.
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
Returned if the
.I mutex
parameter is NULL, or if the value specified by
.I attr
is a valid memory address, but the identifier is not a mutex attributes object.
.IP ENOMEM
The process's heap has become exhausted.
.TP
.SH USAGE EXAMPLE
.PP
See the manual page for the
.B pthread_mutex_lock()
function for an example of how to initialize and destroy a mutex object.
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_mutexattr_init(3), pthread_mutexattr_destroy(3),
.B pthread_mutex_lock(3), pthread_mutex_unlock(3).
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>







