\# $Id: pthread_mutex_lock.man,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_mutex_lock.man,v $
\# Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
\# PCThreads for Linux.
\#
\# Revision 1.1.1.1  1996/06/29 01:20:51  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:52  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:27  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:08  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_mutex_lock \- Acquire (lock) a mutex
.br
pthread_mutex_unlock \- Release (unlock) a mutex
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "pthread_mutex_lock(pthread_mutex_t *" mutex )
.PP
.B int
.br
.BI pthread_mutex_unlock(pthread_mutex_t *" mutex )
.SH DESCRIPTION
A mutex is an abstraction (object) used to provide mutually exclusive, or
serialized,  access to global or static variables.  To guaranteed serialized
access to such data, the application must be programmed such that all threads
that access such data must first acquire or lock mutex guarding the data.  If
the mutex is locked because a previous thread is accessing the variable, any
thread attempting to acquire the mutex will be blocked until the mutex is
released or unlocked.  When subsequently unlocked the mutex will be transferred
to the highest priority thread blocked at the mutex.
.SH PARAMETERS
.SS Input Parameters
.IP mutex
The address of a variable of type
.B pthread_mutex_t.
The parameter must have been previously initialized by a call to
.B pthread_mutex_init().
.SS Return Parameters
.IP None
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
The value specified by one or more of the function's parameters is invalid.
.TP
.IP EDEADLK
The library has detected a deadlock condition.  This is most often seen
when the calling thread attempts to lock a mutex it already has locked.
.TP
.IP ENOLCK
The calling thread has attempted to unlock a mutex that is not currently
locked.
.IP ENOMEM
The process's heap has become exhausted.
.SH USAGE EXAMPLE
.PP
.nf
#include <pthread.h>

static pthread_mutex_t mu;
static long global_variable;

int
main()
{
   int status;


   status = pthread_mutex_init( &mu, NULL );
   CHECK(status, "pthread_mutex_init()");
               .
               .
               .
   status = pthread_mutex_lock( &mu );
   CHECK(status, "pthread_mutex_lock()");

   global_variable += 1;

   status = pthread_mutex_unlock( &mu );
   CHECK(status, "pthread_mutex_unlock()");   
               .
               .
               .
   status = pthread_mutex_destroy( &mu, NULL );
   CHECK(status, "pthread_mutex_destroy()");

   return( EXIT_SUCCESS );
}

.fi
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_lock_global_np(3)
.B pthread_unlock_global_np(3)
.B pthread_mutexinit(3)
.B pthread_mutex_destroy(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>
