\# $Id: pthread_once.man,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_once.man,v $
\# Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
\# PCThreads for Linux.
\#
\# Revision 1.1.1.1  1996/06/29 01:20:51  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:52  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:28  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:08  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_once \- Execute a routine once, and only once.
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "pthread_once(pthread_once_t *" executed ","
.br
.BI "             void (*" init_routine ")(void))"
.SH DESCRIPTION
The
.B pthread_once()
function initializes the
.I executed
parameter and invokes the function pointed to by the
.I init_routine
as a single, atomic operation.  Subsequent calls to
.B pthread_once()
with the same
.I executed
parameter will 
.B NOT
call the function specified by the
.I init_routine
parameter.  If
.B pthread_once()
returns successfully, the initialization routine is guaranteed to have
completed.
.sp
For example, programmers place code that initializes global variables,
mutexes, per-thread keys, etc., into an intialization routine that may
then be passed to the
.B pthread_once()
function as the
.I init_routine
parameter.
.SH PARAMETERS
.SS Input Parameters
.IP init_routine 
A pointer to a function that initializes an application's global data.  This
routine is called once, and only once, no matter how many times the 
.B pthread_once()
function is invoked.
.SS Return Parameters
.IP executed
The address of a variable whose value indicates whether the function specified
by
.I init_routine
has been executed.  Each one-time initialization routine must have its own
unique
.I executed
variable.
.SH RETURN VALUE
A value of zero (0) is returned if no errors are encountered.
When an error condition arises, this routine returns an integer value 
indicating the reason for the error.  Possible error values are:
.TP
.IP EINVAL
The value specified by one or more of the parameters is invalid.
.IP ENOMEM
The process's heap has become exhausted.
.SH USAGE EXAMPLE
.PP
See the manual page for the
.B pthread_key_create()
routine.
.PP
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_key_create()
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>







