\# $Id: pthread_self.man,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_self.man,v $
\# Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
\# PCThreads for Linux.
\#
\# Revision 1.1.1.1  1996/06/29 01:20:51  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:52  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:28  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:09  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1.1.1  1995/12/16 21:41:59  mtp
\# Placed under CVS control
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_self \- Return the identifier of the calling thread
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B pthread_t
.br
.B pthread_self( void )
.SH DESCRIPTION
The
.B pthread_self()
function enables the calling thread to obtain its own identifier.  In
.I PCthreads,
all objects are identified using opaque data types, called handles.  For 
threads, the handle type is
.BI pthread_t.
All object handles are declared as forward pointers to structures that
are defined by the implementation.
.SH PARAMETERS
.SS Input Parameters
None
.SS Return Parameters
None
.SH RETURN VALUE
The
.pthread_self()
function returns the identifier of the calling thread.  This function always
succeeds.
.SH USAGE EXAMPLE
.PP
Create a thread that prints the address of its identifier.
.PP
.nf
#include <stdlib.h>
#include <pthread.h>

void proc( void )
{
    /*
	 * Lock the global lock to prevent reentering the
	 * non-reentrant function, printf().
	 */
	pthread_global_lock_np();
	printf("Child thread's handle = %x\n", pthread_self() );
	pthread_global_unlock_np();

	pthread_exit( (void *) SUCCESS );
}

int
main()
{
	pthread_t child;

	(void) pthread_create( &child, NULL, (thread_proc_t) proc, NULL );
	(void) pthread_join( child, &exit_status );


	return( EXIT_SUCCESS );
}
.fi
.SH CONFORMING TO
POSIX .1c
.SH SEE ALSO
.B pthread_create(3)
.SH AUTHOR
Michael T. Peterson <mtp@zso.dec. mtp@big.aa.net>







