\# $Id: pthread_setcancelstate.man,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $
\#---------------------------------------------------------------------------
\#    
\#  COPYRIGHT NOTICE
\#    
\#  Copyright (C) 1995, 1996 Michael T. Peterson
\#  This file is part of the PCthreads (tm) multithreading library
\#  package.
\#    
\#  The source files and libraries constituting the PCthreads (tm) package
\#  are free software; you can redistribute them and/or modify them under
\#  the terms of the GNU Library General Public License as published by the Free
\#  Software Foundation; either version 2 of the License, or (at your
\#  option) any later version.
\#    
\#  The PCthreads (tm) package is distributed in the hope that it will
\#  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
\#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
\#  Library General Public License for more details.
\#    
\#  You should have received a copy of the GNU Library General Public
\#  License along with this library (see the file COPYING.LIB); if not,
\#  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
\#  MA 02139, USA.
\#    
\#  To report bugs, bug fixes, or provide feedback you may contact the
\#  author, Michael T. Peterson, at either of the two email addresses listed
\#  below:
\#    
\#             mtp@big.aa.net (preferred)
\#             mtp@zso.dec.com
\#    
\#    
\#  Michael T. Peterson
\#  Redmond, WA.
\#  13 January, 1996
\#---------------------------------------------------------------------------
\# $Log: pthread_setcancelstate.man,v $
\# Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
\# PCThreads for Linux.
\#
\# Revision 1.1.1.1  1996/06/29 01:20:51  mtp
\# pthreads 1.0 distribution
\#
\# Revision 1.1.1.1  1996/06/28 15:45:40  mtp
\# Official V1.0 Sources
\#
\# Revision 2.0  1996/06/28 14:57:53  mtp
\# + Release 2.0
\#
\# Revision 1.3  1996/01/14 20:22:28  mtp
\# + Modified copyright and permission notice to conform with GNU requirements.
\#
\# Revision 1.2  1996/01/14 18:32:09  mtp
\# + Replaced @Header@ with COPYRIGHT NOTICE.
\#
\# Revision 1.1  1996/01/10 19:01:09  mtp
\# + Placed under CVS control.
\#
\#---------------------------------------------------------------------------
.TH PCTHREADS 3 "July, 1995" PCTHREADS "POSIX Threads V1.0 For Linux"
.SH NAME
pthread_setcancelstate \- Set thread's cancellation state.
.sp
pthread_setcanceltype \- Set thread's cancellation type.
.SH SYNOPSIS
.B #include <pthread.h>
.sp
.B int
.br
.BI "pthread_setcancelstate(int " new_state ","
.br
.BI "                       int * " prev_state )

.br
.BI "pthread_setcanceltype(int " new_type ","
.br
.BI "                      int * " prev_type )
.SH DESCRIPTION
These two functions allow a thread to control its cancellation state and its
cancellation type, respectively.  The cancellation state of a thread determines
whether a cancel can be delivered to the thread.  If the cancellation state has
a value of PTHREAD_CANCEL_ENABLE, then it is eligible for cancellation and may
accept cancel requests.  If the value is PTHREAD_CANCEL_DISABLE, then the thread
is not eligible for cancellation and the cancel request will remain pending.  

.sp 
The cancellation type of a thread allows a thread to control how it may be
canceled.  If the thread's cancellation type is PTHREAD_CANCEL_DEFERRED, cancels
will only be delivered during a call to one of the defined cancellation points
(see 
.B pthread_cancel()
).  If the cancellation type is PTHREAD_CANCEL_ASYNCHRONOUS, a cancel may be
delivered at any time, not just during the execution of a cancellation point
function.  

.SH
PARAMETERS 
.SS Input Parameters 
.IP new_state
The value to which the cancellation state of the thread is to be set.  The only
allowable values are PTHREAD_CANCEL_ENABLE and PTHREAD_CANCEL_DISABLE.

.IP new_type
The value to which the cancellation type of the thread is to be set.  The only
allowable values are PTHREAD_CANCEL_DEFERRED and PTHREAD_CANCEL_ASYNCHRONOUS.

.SS Return Parameters 
.IP prev_state
The address of the value of the calling thread's cancellation state prior to the
completion of
.B pthread_setcancelstate().

.IP prev_type
The address of the value of the calling thread's cancellation type prior to the
completion of
.B pthread_setcanceltype().
.SH RETURN VALUE
A value
of zero (0) is returned if no errors are encountered.  When an error condition
arises, this routine returns an integer value indicating the reason for the
error.  Possible error values are:  
.TP 
.IP EINVAL
The value specified by one or more of the parameters is invalid.  
.TP 
.IP ENOMEM
The process's heap has become exhausted.  
.SH USAGE EXAMPLE 
.PP 
see the man page for 
.B pthread_cancel().
.SH CONFORMING TO POSIX .1c 
.SH SEE ALSO 
.B pthread_cancel(), pthread_testcancel()
.SH AUTHOR 
Michael T. Peterson <mtp@zso.dec.com,  mtp@big.aa.net>

