#ifndef lint
static const char rcsid[] = "$Id: clock.c,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Redmond, WA.
 *  13 January, 1996
 ****************************************************************************
 * $Log: clock.c,v $
 * Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
 * PCThreads for Linux.
 *
 * Revision 1.1.1.1  1996/06/29 01:20:51  mtp
 * pthreads 1.0 distribution
 *
 * Revision 1.1.1.1  1996/06/28 15:45:41  mtp
 * Official V1.0 Sources
 *
 * Revision 2.0  1996/06/28 14:57:59  mtp
 * + Release 2.0
 *
 * Revision 1.3  1996/01/14 20:22:40  mtp
 * + Modified copyright and permission notice to conform with GNU requirements.
 *
 * Revision 1.2  1996/01/14 18:32:27  mtp
 * + Replaced @Header@ with COPYRIGHT NOTICE.
 *
 * Revision 1.1.1.1  1995/12/16 21:41:59  mtp
 * Placed under CVS control
 *
 ****************************************************************************
 */
#include <stdlib.h>
#include <stdio.h>
#include "private.h"
#include "clock.h"
#include "machdep.h"

#ifdef sigprocmask
#undef sigprocmask
#endif

extern void nub_timer_intr( int signum );

/*
 * Set the interval timer to fire periodically. The interval is determined
 * at system startup.  See init.c.
 */
static struct itimerval    pthread_interval_timer;
static struct itimerval    process_interval_timer;

static sigset_t            timer_sigmask, process_sigmask;

void
install_interval_timer( long timer_interval )
{
   struct sigaction pthread_sigaction;

   /*
    * Initialize the interval timer interval structure.
    */
   pthread_interval_timer.it_interval.tv_sec = 0;
   pthread_interval_timer.it_interval.tv_usec = timer_interval;

   pthread_interval_timer.it_value.tv_sec = 0;
   pthread_interval_timer.it_value.tv_usec = timer_interval;

   sigemptyset( &timer_sigmask );
   sigaddset( &timer_sigmask, SIGVTALRM );

   sigemptyset( &process_sigmask );

   /*
    *  --  Install 'nub_timer_intr' to be invoked everytime a 
    *      SIGVTALRM signal occurs.
    */
   sigemptyset( &pthread_sigaction.sa_mask );
   pthread_sigaction.sa_handler = nub_timer_intr;
   pthread_sigaction.sa_flags = (SA_RESTART);
   sigaction( SIGVTALRM, &pthread_sigaction, NULL );

   /*
    *  --  initialize the interval timer so that it sends a SIGVTALRM signal
    *      after the specified number of microseconds have elapsed.
    */
   setitimer( ITIMER_VIRTUAL, 
              &pthread_interval_timer, 
              &process_interval_timer );
}

void
reset_interval_timer( void )
{
   (void) sigprocmask( SIG_UNBLOCK, &timer_sigmask, &process_sigmask );
}
