#ifndef lint
static const char rcsid[] = "$Id: config.c,v 1.1.1.1 1997/02/05 22:44:56 netcrep Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Redmond, WA.
 *  13 January, 1996
 ****************************************************************************
 * $Log: config.c,v $
 * Revision 1.1.1.1  1997/02/05 22:44:56  netcrep
 * PCThreads for Linux.
 *
 * Revision 1.1.1.1  1996/06/29 01:20:51  mtp
 * pthreads 1.0 distribution
 *
 * Revision 1.1.1.1  1996/06/28 15:45:41  mtp
 * Official V1.0 Sources
 *
 * Revision 2.0  1996/06/28 14:58:00  mtp
 * + Release 2.0
 *
 * Revision 1.3  1996/01/14 20:22:41  mtp
 * + Modified copyright and permission notice to conform with GNU requirements.
 *
 * Revision 1.2  1996/01/14 18:32:28  mtp
 * + Replaced @Header@ with COPYRIGHT NOTICE.
 *
 * Revision 1.1.1.1  1995/12/16 21:42:00  mtp
 * Placed under CVS control
 *
 ****************************************************************************
 */
#include <string.h>
#include <sys/stat.h>
#include <private.h>
#include "config.h"

extern void parse_config( FILE *fd, config_t *config );

static const char *rc_file = ".pthread_initrc";
static char home_dir[BUFSIZ + 1];
static char current_dir[BUFSIZ + 1];
static char tmp_file[BUFSIZ + 1];

static int
file_exists( const char *filename )
{
   struct stat buf;

   return((stat( filename, &buf ) < 0 ? 0 : buf.st_mode ));
}

static void
x_init_config( const char *config_file, config_t *config )
{
   FILE *fd;

   fd = fopen( config_file, "r+" );

   parse_config( fd, config );

   fclose( fd );
}

void
init_config( config_t *config )
{
   char *config_file = NULL;

   getcwd( current_dir, BUFSIZ );
   strcpy( home_dir, getenv( "HOME" ) );

   sprintf( tmp_file, "%s/%s", current_dir, rc_file );
   if( file_exists( tmp_file ) )
       config_file = tmp_file;
   else
   {
       sprintf( tmp_file, "%s/%s", home_dir, rc_file );
       if( file_exists( tmp_file ) )
           config_file = tmp_file;
   }

   /*
    * Set the default values.
    */
   config->timer_interval = TIMER_C_INTERVAL_USECS;
   config->timer_idle_interval = TIMER_C_IDLE_INTERVAL;
   config->default_quantum = PTHREAD_DEFAULT_QUANTUM_C;
   config->default_sched_policy = PTHREAD_DEFAULT_SCHED_POLICY_C;
   config->default_thread_prio = PTHREAD_DEFAULT_PRIO_C;
   config->default_stack_size = PTHREAD_DEFAULT_STACKSIZE_C;

   if( config_file == NULL )
       return;

   x_init_config( config_file, config );

   /*
    * Make sure the new config structure is appropriate.
    */
   if( config->timer_interval < 10000 )
       config->timer_interval = TIMER_C_INTERVAL_USECS;

   if(config->timer_idle_interval < config->timer_interval )
       config->timer_idle_interval = config->timer_interval;

   if( config->default_thread_prio < PTHREAD_MIN_PRIO_C )
       config->default_thread_prio = PTHREAD_MIN_PRIO_C;

   if( config->default_thread_prio > PTHREAD_MAX_PRIO_C )
       config->default_thread_prio = PTHREAD_MAX_PRIO_C;
}
