#ifndef lint
static const char rcsid[] = "$Id: sigwait.c,v 1.2 1997/03/19 18:25:10 netcrep Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Redmond, WA.
 *  13 January, 1996
 ****************************************************************************
 * $Log: sigwait.c,v $
 * Revision 1.2  1997/03/19 18:25:10  netcrep
 * Rename library from pthread to pcthreads.
 *
 * Revision 1.1.1.1  1997/02/05 22:44:57  netcrep
 * PCThreads for Linux.
 *
 * Revision 1.1.1.1  1996/06/29 01:20:52  mtp
 * pthreads 1.0 distribution
 *
 * Revision 1.1.1.1  1996/06/28 15:45:41  mtp
 * Official V1.0 Sources
 *
 * Revision 2.0  1996/06/28 14:58:06  mtp
 * + Release 2.0
 *
 * Revision 1.3  1996/01/14 20:22:46  mtp
 * + Modified copyright and permission notice to conform with GNU requirements.
 *
 * Revision 1.2  1996/01/14 18:32:34  mtp
 * + Replaced @Header@ with COPYRIGHT NOTICE.
 *
 * Revision 1.1.1.1  1995/12/16 21:42:00  mtp
 * Placed under CVS control
 *
 ****************************************************************************
 */
#include "pcthread.h"
#include "sigwait.h"
#include "machdep.h"
#include "wait_block.h"

/*
 * This data structure realizes the global list of wait blocks.  Each wait block
 * may have one or more waiting threads.
 */
static sigwait_block_list_t    waitblock_list;
static pthread_once_t          initialized = PTHREAD_ONCE_INIT;

static void 
init_sigwait_list( void )
{
   pthread_mutex_init( &waitblock_list.lock, NULL );
}

/*
 * sig_wait_thread() is called by the POSIX .1c sigwait() service.
 */
int
sig_wait_thread( sigset_t sigset, struct PTHREAD_HANDLE *active_th )
{
   sigwait_block_t *wb;
   int sig, status;

   status = pthread_once( &initialized, init_sigwait_list );
   if( status != SUCCESS )
      return( status );

   /*
    * If a wait block exists whose set of signals is equal to this
    * thread's signal set, have the thread wait at this wait block
    * (i.e., a pointer to the matching waitblock is returned).  
    * Otherwise, if a waitblock matching the sigset was not found, create, 
    * initialize, and insert a new wait block into the global list of 
    * wait blocks.
    */
   if( (wb = waitblock_get_matching( &waitblock_list, sigset )) == NULL )
   {
       wb = init_sigwait_block();
       wb->wb_sigset = sigset;
       insert_waitblock( &waitblock_list, wb );
   }

   /*
    * Execute a condition wait until a signal is delivered (the list's 
    * signal_pending flag should be TRUE).  When signaled, the waiting 
    * thread will clear the signal pending flag, remove itself from any 
    * other signal list on which it was waiting, and return.
    */
   pthread_mutex_lock( &wb->wb_lock );
   while( wb->signal_pending == 0 )
       pthread_cond_wait( &wb->wb_cond, &wb->wb_lock );

   sig = wb->signal_pending;
   wb->signal_pending = 0;
   pthread_mutex_unlock( &wb->wb_lock );

   return( sig );
}


int
sig_unwait_thread( int sig )
{
    sigwait_block_t *wb;
    int status;

    status = pthread_once( &initialized, init_sigwait_list );
    if( status != SUCCESS )
       return( status );

    /*
     * Find a wait block containing a thread waiting for the specified signal.
     * Return FALSE if a wait block isn't found.
     */
    if( (wb = get_waitblock( &waitblock_list, sig )) == NULL )
       return( FALSE );

    /*
     * OK, signal the condition variable.  This will cause the highest
     * priority thread waiting at the condition variable to be put on the
     * ready queue.
     */
    pthread_mutex_lock( &wb->wb_lock);
    wb->signal_pending = sig;
    pthread_mutex_unlock( &wb->wb_lock );
    pthread_cond_signal( &wb->wb_cond );

    return( TRUE );
}

