#ifndef lint
static const char rcsid[] = "$Id: pthread_mutexattr.c,v 1.2 1997/03/19 18:25:13 netcrep Exp $";
#endif
/****************************************************************************
 *    
 *  COPYRIGHT NOTICE
 *    
 *  Copyright (C) 1995, 1996 Michael T. Peterson
 *  This file is part of the PCthreads (tm) multithreading library
 *  package.
 *    
 *  The source files and libraries constituting the PCthreads (tm) package
 *  are free software; you can redistribute them and/or modify them under
 *  the terms of the GNU Library General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your
 *  option) any later version.
 *    
 *  The PCthreads (tm) package is distributed in the hope that it will
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *    
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library (see the file COPYING.LIB); if not,
 *  write to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
 *  MA 02139, USA.
 *    
 *  To report bugs, bug fixes, or provide feedback you may contact the
 *  author, Michael T. Peterson, at either of the two email addresses listed
 *  below:
 *    
 *             mtp@big.aa.net (preferred)
 *             mtp@zso.dec.com
 *    
 *    
 *  Michael T. Peterson
 *  Redmond, WA.
 *  13 January, 1996
 ****************************************************************************
 * $Log: pthread_mutexattr.c,v $
 * Revision 1.2  1997/03/19 18:25:13  netcrep
 * Rename library from pthread to pcthreads.
 *
 * Revision 1.1.1.1  1997/02/05 22:44:57  netcrep
 * PCThreads for Linux.
 *
 * Revision 1.1.1.1  1996/06/29 01:20:54  mtp
 * pthreads 1.0 distribution
 *
 * Revision 1.1.1.1  1996/06/28 15:45:41  mtp
 * Official V1.0 Sources
 *
 * Revision 2.0  1996/06/28 14:58:15  mtp
 * + Release 2.0
 *
 * Revision 1.3  1996/01/14 20:22:56  mtp
 * + Modified copyright and permission notice to conform with GNU requirements.
 *
 * Revision 1.2  1996/01/14 18:32:46  mtp
 * + Replaced @Header@ with COPYRIGHT NOTICE.
 *
 * Revision 1.1.1.1  1995/12/16 21:42:00  mtp
 * Placed under CVS control
 *
 ****************************************************************************
 */
#include "pcthread.h"
#include "public.h"
#include "mutex_attr.h"
#include "error.h"

extern mutex_attr_obj_t *  get_default_mu_attr_obj( void );

int  
pthread_mutexattr_init( pthread_mutexattr_t  *handle )
{
   int flag, st;
   struct PTHREAD_MUTEXATTR_HANDLE *attr_h;

   if( (st = system_init()) == FAILURE )
       return( st );

   sys_disable_ctxsw( flag );
   if( handle == NULL )
   {
       set_thread_errno( EINVAL );
       RETURN( EINVAL );
   }

   attr_h = allocate_handle( MUTEX_ATTR_C );
   if( attr_h == NULL )
   {
       set_thread_errno( ENOMEM );
       RETURN( ENOMEM );
   }

   attr_h->type = MUTEX_ATTR_C;
   attr_h->obj = get_default_mu_attr_obj();

   *handle = attr_h;
   RETURN( SUCCESS );
}

int  
pthread_mutexattr_destroy( pthread_mutexattr_t  *handle )
{
   int flag, st;

   if( (st = system_init()) == FAILURE )
       return( st );

   sys_disable_ctxsw( flag );
   if( handle == NULL || (*handle)->type != MUTEX_ATTR_C )
   {
       set_thread_errno( EINVAL );
       RETURN( EINVAL );
   }

   destroy_mu_attr_obj( (*handle)->obj );
   free_handle( *handle );
   handle = NULL;

   RETURN( SUCCESS );
}

int  
pthread_mutexattr_setprotocol(pthread_mutexattr_t  *handle, 
                              pthread_protocol_t protocol )
{
   int flag, st;

   if( (st = system_init()) == FAILURE )
       return( st );

   sys_disable_ctxsw( flag );
   if( handle == NULL || (*handle)->type != MUTEX_ATTR_C )
   {
       set_thread_errno( EINVAL );
       RETURN( EINVAL );
   }

   if( protocol != SCHED_MUTEX_NO_PRIO_INHERIT_C ||
       protocol != SCHED_MUTEX_PRIO_INHERIT_C ||
       protocol != SCHED_MUTEX_PRIO_PROTECT_C )
   {
       set_thread_errno( EINVAL );
       RETURN( EINVAL );
   }

   (*handle)->obj->mu_sched_protocol = protocol;
   RETURN( ENOSYS );
}

int
pthread_mutexattr_getprotocol( pthread_mutexattr_t *handle,
                              pthread_protocol_t *protocol )
{
   int flag, st;

   if( (st = system_init()) == FAILURE )
       return( st );

   sys_disable_ctxsw( flag );
   if( handle == NULL || (*handle)->type != MUTEX_ATTR_C )
   {
       set_thread_errno( EINVAL );
       RETURN( EINVAL );
   }

   *protocol = (*handle)->obj->mu_sched_protocol;
   RETURN( ENOSYS );
}

int
pthread_mutexattr_getprio_ceiling( pthread_mutexattr_t *handle,
                                   int *prio_ceiling )
{
   int flag, st;

   if( (st = system_init()) == FAILURE )
       return( st );

   sys_disable_ctxsw( flag );
   if( handle == NULL || (*handle)->type != MUTEX_ATTR_C )
   {
       set_thread_errno( EINVAL );
       RETURN( EINVAL );
   }

   RETURN( ENOSYS );
}

int 
pthread_mutexattr_setprio_ceiling( pthread_mutexattr_t  *handle,
                                   int prio_ceiling )
{
   int flag, st;

   if( (st = system_init()) == FAILURE )
       return( st );

   sys_disable_ctxsw( flag );
   if( handle == NULL || (*handle)->type != MUTEX_ATTR_C )
   {
       set_thread_errno( EINVAL );
       RETURN( EINVAL );
   }

   RETURN( ENOSYS );
}
